<?php

declare(strict_types = 1);

namespace Drupal\virtual_select\Element;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\FormElement;

/**
 * Defines a form element implementation for the Virtual Select library.
 *
 * @FormElement("virtual_select")
 */
class VirtualSelect extends FormElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo(): array {
    $class = static::class;
    return [
      '#input' => TRUE,
      '#multiple' => FALSE,
      '#process' => [
        [$class, 'processElement'],
      ],
      '#theme' => 'virtual_select',
      '#theme_wrappers' => ['form_element'],
      '#search' => TRUE,
      '#show_value_as_tags' => TRUE,
      '#show_selected_options_first' => TRUE,
      '#placeholder' => $this->t('- Select -'),
      '#search_placeholder' => $this->t('Search...'),
      '#no_options_text' => $this->t('No options found.'),
    ];
  }

  /**
   * Processes the virtual_select element.
   *
   * @param array $element
   *   The element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param array $complete_form
   *   The complete form.
   *
   * @return array
   *   The processed element.
   */
  public static function processElement(array &$element, FormStateInterface $form_state, array &$complete_form): array {
    $id = Html::getUniqueId('virtual-select');

    $multiple = $element['#multiple'] ?? FALSE;
    $element['#tree'] = TRUE;
    $element['#attributes']['id'] = $id;
    $element['#attributes']['class'][] = 'virtual-select';
    $element['#attached']['library'][] = 'virtual_select/virtual_select';

    foreach ($element['#options'] ?? [] as $value => $label) {
      $element['#attached']['drupalSettings'][$id]['options'][] = [
        'label' => $label,
        'value' => $value,
      ];
    }

    $element['#attached']['drupalSettings'][$id]['selected'] = $element['#default_value'] ?? [];
    $element['#attached']['drupalSettings'][$id]['multiple'] = $multiple;
    $element['#attached']['drupalSettings'][$id]['search'] = $element['#search'];
    $element['#attached']['drupalSettings'][$id]['show_value_as_tags'] = $element['#show_value_as_tags'];
    $element['#attached']['drupalSettings'][$id]['show_selected_options_first'] = $element['#show_selected_options_first'];
    $element['#attached']['drupalSettings'][$id]['placeholder'] = $element['#placeholder'];
    $element['#attached']['drupalSettings'][$id]['search_placeholder'] = $element['#search_placeholder'];
    $element['#attached']['drupalSettings'][$id]['no_options_text'] = $element['#no_options_text'];

    $element['input'] = [
      '#type' => 'select',
      '#options' => $element['#options'] ?? [],
      '#empty_option' => '',
      '#default_value' => $element['#default_value'] ?? [],
      '#multiple' => $multiple,
      '#parents' => [$element['#name'] . '-input'],
      '#attributes' => [
        'data-virtual-select' => $id,
        'class' => ['hidden'],
      ],
      '#wrapper_attributes' => [
        'class' => ['hidden'],
      ],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function valueCallback(&$element, $input, FormStateInterface $form_state): mixed {
    $fieldName = $element['#name'] . '-input';
    $form_state->addCleanValueKey($element['#name'] . '-input');
    return $form_state->getUserInput()[$fieldName] ?? NULL;
  }

}
