<?php

declare(strict_types = 1);

namespace Drupal\virtual_select\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\OptionsWidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides the Virtual Select widget.
 *
 * @FieldWidget(
 *   id = "virtual_select",
 *   label = @Translation("Virtual Select"),
 *   field_types = {
 *     "entity_reference",
 *     "list_string"
 *   },
 *   multiple_values = TRUE
 * )
 */
class VirtualSelect extends OptionsWidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    $element += [
      '#type' => 'virtual_select',
      '#options' => $this->getOptions($items->getEntity()),
      '#default_value' => $this->getSelectedOptions($items),
      '#multiple' => $this->multiple && count($this->options) > 1,
    ];

    return $element;
  }

}
